-----------------------------------------------------------------------------

local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

local Dewdrop = AceLibrary:HasInstance("Dewdrop-2.0") and AceLibrary("Dewdrop-2.0")
local AssessmentDisplaySets = Assessment:NewModule("DisplaySets", "AceEvent-2.0")
AssessmentDisplaySets.db = Assessment:AcquireDBNamespace("DisplaySets")
local profileDefaults =
{
	displaySets =
	{
		-- Defaults are inserted here in OnInitialize
	}
}

-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentDisplaySets")
L:RegisterTranslations("enUS", function() return {
	["Assessment"] = true,

	["Display Sets"] = true,
	["Display set options."] = true,

	["Add New Display Set"] = true,
	["Adds a new display set to the user interface."] = true,
	["Enter a new display set name"] = true,
	["Enter the name to copy this display set to"] = true,

	["Options for display set %s"] = true,

	["List Type"] = true,
	["Select a list type that controls what is displayed."] = true,
	["Bar Value Tag"] = true,
	["Select how to calculate bar values."] = true,
	["Sort Key Tag"] = true,
	["Select how to sort the bars."] = true,
	["Descending Sort"] = true,
	["Enable to use descending sort. Disable to use ascending."] = true,

	["Favorite"] = true,
	["Show this display set as an option when clicking on a window."] = true,
	["Selectable"] = true,
	["Make this display set selectable from window menus."] = true,
	
	["Title Tag"] = true,
	["Select what to show for the window's title."] = true,

	["Filter Type"] = true,
	["Select how to filter the list."] = true,
	["Filter Tag"] = true,
	["Advanced filter type setting."] = true,

	["Fight Data"] = true,
	["Select which fight data set to display."] = true,
	["Total Data"] = true,
	["Current Fight"] = true,
	["Previous Fight"] = true,

	["Text"] = true,
	["Text options."] = true,
	["Add New Text"] = true,
	["Adds a new text to the display set."] = true,

	["Text %d"] = true,
	["Options for text %d"] = true,

	["Text Tag"] = true,
	["Select how to display text."] = true,

	["Alpha"] = true,
	["Set the text alpha."] = true,

	["Alignment"] = true,
	["Select how to align the text."] = true,
	["Left"] = true,
	["Center"] = true,
	["Right"] = true,

	["Font"] = true,
	["Font for text."] = true,

	["Font Size"] = true,
	["Font size for text."] = true,

	["Font Shadow"] = true,
	["Show font shadow on text"] = true,
	["Outline"] = true,
	["Sets the outline for text"] = true,

	["Offsets"] = true,
	["Set text offsets."] = true,
	["Left"] = true,
	["Set the left offset for the text."] = true,
	["Right"] = true,
	["Set the right offset for the text."] = true,
	["Vertical"] = true,
	["Set the vertical offset for the text."] = true,

	["Click Options"] = true,
	["Click options."] = true,
	["Action"] = true,
	["Specify what to do when a bar is clicked."] = true,
	["Ignore"] = true,
	["Show Display Set"] = true,
	["Display Set"] = true,
	["Select a display set to show when the bar is clicked."] = true,
	["Display Set Tag"] = true,
	["Advanced display set specification."] = true,
	["Data"] = true,
	["On click data."] = true,

	["Default Color"] = true,
	["Change the default bar color."] = true,
	["Remove"] = true,
	["Remove this text."] = true,

	["Remove"] = true,
	["Remove this display set."] = true,
	["This display set cannot be removed currently because the display set %s references it for on-click."] = true,
	["Copy"] = true,
	["Copy this display set."] = true,
	
	["Any Tag String"] = true,
	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,
	
	["Display Set List"] = true,

	["Set All"] = true,
	["Set all display set options at once."] = true,
} end)

-----------------------------------------------------------------------------

Assessment.Options.args.uiSpacer =
{
	type = 'header',
	order = 50,
}

Assessment.Options.args.displaySets =
{
	type = "group",
	name = L["Display Sets"],
	desc = L["Display set options."],
	order = 60,
	args = { },
}

-----------------------------------------------------------------------------

local get = AssessmentDisplaySets.GetOptionValue
local set = AssessmentDisplaySets.SetOptionValue

local function displaySetGet(passValue)
	return get(AssessmentDisplaySets.db.profile.displaySets[passValue[1]], passValue[2])
end

local function displaySetSet(passValue, value)
	set(AssessmentDisplaySets.db.profile.displaySets[passValue[1]], passValue[2], value)
	AssessmentDisplaySets:UpdateDisplaySets(passValue[1])
end

local function textGet(passValue)
	return get(AssessmentDisplaySets.db.profile.displaySets[passValue[1][1]].text[passValue[1][2]], passValue[2])
end

local function textSet(passValue, value)
	set(AssessmentDisplaySets.db.profile.displaySets[passValue[1][1]].text[passValue[1][2]], passValue[2], value)
	AssessmentDisplaySets:UpdateDisplaySets(passValue[1][1])
end

local function displaySetRemove(passValue)
	AssessmentDisplaySets:RemoveDisplaySet(passValue[1])
end

local function displaySetCopy(passValue)
	AssessmentDisplaySets:CopyDisplaySet(passValue[1])
end

local setAllTextValues =
{
	font		= "",
	fontSize	= 9,
	shadow		= true,
	outline		= "",
}

local function setAllTextGet(passValue)
	return setAllTextValues[passValue]
end

local function setAllTextSet(passValue, value)
	if setAllTextValues[passValue] == value then return end
	setAllTextValues[passValue] = value
	
	for k,v in pairs(AssessmentDisplaySets.db.profile.displaySets) do
		for i = 1,v.text.count do
			v.text[i][passValue] = value
		end
		AssessmentDisplaySets:UpdateDisplaySets(k)
	end
end

-----------------------------------------------------------------------------

local fullDisplaySetValidate	= { }
local listTypeValidate			= { }
local filterTypeValidate		= { }

local displaySetsArgs =
{
	setAll =
	{
		type = "group",
		name = L["Set All"],
		desc = L["Set all display set options at once."],
		order = -2,
		args =
		{
			font =
			{
				type = "text",
				name = L["Font"],
				desc = L["Font for text."],
				get = setAllTextGet,
				set = setAllTextSet,
				passValue = "font",
				validate = SML_font,
			},
			fontSize =
			{
				type = "range",
				name = L["Font Size"],
				desc = L["Font size for text."],
				min = 4,
				max = 32,
				step = 1,
				get = setAllTextGet,
				set = setAllTextSet,
				passValue = "fontSize",
			},
			shadow =
			{
				type = "toggle",
				name = L["Font Shadow"],
				desc = L["Show font shadow on text"],
				get = setAllTextGet,
				set = setAllTextSet,
				passValue = "shadow",
			},
			outline =
			{
				type = "text",
				name = L["Outline"],
				desc = L["Sets the outline for text"],
				get = setAllTextGet,
				set = setAllTextSet,
				passValue = "outline",
				validate = { [""] = L["None"], OUTLINE = L["Normal"], THICKOUTLINE = L["Thick"] },
			},
		}
	},
	add =
	{
		type = "execute",
		name = L["Add New Display Set"],
		desc = L["Adds a new display set to the user interface."],
		func = function() AssessmentDisplaySets:AddDisplaySet() end,
		order = -1
	},
}

local defaultDisplaySetProfile =
{
	titleTag		= "Assessment",
	
	listType		= "GroupMembers", 
	barValueTag		= "damageDealt", 
	sortKeyTag		= "barValue",
	filterTag		= "1",
	descendingSort	= true,
	fightIndex		= 0,
	colorTypeList	= { "Class", count = 1 },
	defaultColor	= { 0.3, 0.45, 0.75 },
	favorite		= true,
	selectable		= true,
	
	onClickType		= "NONE",		-- NONE, or PUSH_DISPLAY_SET currently.
--	onClickType		= "PUSH_DISPLAY_SET",
--  onClickDisplaySet = "Display Set Name",
--	onClickData		= "EnumerateTable|damageDealt|[key]|attackDetails",
	
	text =
	{
		count = 2,
		{
			alignment	= "LEFT",
			font		= "Arial Narrow",
			fontSize	= 9,
			outline		= "",
			shadow		= true,
			alpha		= 1,
			tag			= "[rank]. [key]",
			offsets =
			{
				left		= 0,
				right		= -20,
				vertical	= 0,
			},
		},
		{
			alignment	= "RIGHT",
			font		= "Arial Narrow",
			fontSize	= 9,
			outline		= "",
			shadow		= true,
			alpha		= 1,
			tag			= "[barValue]",
			offsets =
			{
				left		= 0,
				right		= 0,
				vertical	= 0,
			},
		},
	},
}

local defaultTextProfile =
{
	alignment	= "LEFT",
	font		= "Arial Narrow",
	fontSize	= 9,
	outline		= "",
	shadow		= true,
	alpha		= 1,
	tag			= "text",
	offsets =
	{
		left		= 0,
		right		= 0,
		vertical	= 0,
	},
}

local displaySetOptionsTemplate =
{
	type = "group",
--	name = ["%s"] = true,
--	desc = ["Options for display set %s"] = true,
	args =
	{
		listType =
		{
			type = "text",
			name = L["List Type"],
			desc = L["Select a list type that controls what is displayed."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "listType",
			validate = listTypeValidate,
			order = 1,
		},
		barValueTag =
		{
			type = "text",
			name = L["Bar Value Tag"],
			desc = L["Select how to calculate bar values."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "barValueTag",
			usage = L["Any Tag String"],
		},
		sortKeyTag =
		{
			type = "text",
			name = L["Sort Key Tag"],
			desc = L["Select how to sort the bars."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "sortKeyTag",
			usage = L["Any Tag String"],
		},
		titleTag =
		{
			type = "text",
			name = L["Title Tag"],
			desc = L["Select what to show for the window's title."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "titleTag",
			usage = L["Any Tag String"],
		},
		filterType =
		{
			type = "text",
			name = L["Filter Type"],
			desc = L["Select how to filter the list."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "filterTag",
			validate = filterTypeValidate,
		},
		filterTag =
		{
			type = "text",
			name = L["Filter Tag"],
			desc = L["Advanced filter type setting."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "filterTag",
			usage = L["Any Tag String"],
		},
		descendingSort =
		{
			type = "toggle",
			name = L["Descending Sort"],
			desc = L["Enable to use descending sort. Disable to use ascending."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "descendingSort",
		},
		fightData =
		{
			type = "text",
			name = L["Fight Data"],
			desc = L["Select which fight data set to display."],
			get = function(passValue) return tostring( displaySetGet(passValue) ) end,
			set = function(passValue, value) displaySetSet(passValue, tonumber(value)) end,
			passValue = "fightIndex",
			validate = { ["0"] = L["Total Data"], ["1"] = L["Current Fight"], ["2"] = L["Previous Fight"] },
		},
		click =
		{
			type = "group",
			name = L["Click Options"],
			desc = L["Click options."],
			args =
			{
				action =
				{
					type = "text",
					name = L["Action"],
					desc = L["Specify what to do when a bar is clicked."],
					get = displaySetGet,
					set = displaySetSet,
					passValue = "onClickType",
					validate = { NONE = L["Ignore"], PUSH_DISPLAY_SET = L["Show Display Set"] }
				},
				displaySet =
				{
					type = "text",
					name = L["Display Set"],
					desc = L["Select a display set to show when the bar is clicked."],
					disabled = function(passValue) return AssessmentDisplaySets.db.profile.displaySets[passValue[1]].onClickType ~= "PUSH_DISPLAY_SET" end,
					get = displaySetGet,
					set = displaySetSet,
					passValue = "onClickDisplaySet",
					validate = fullDisplaySetValidate,
				},
				displaySetTag =
				{
					type = "text",
					name = L["Display Set Tag"],
					desc = L["Advanced display set specification."],
					disabled = function(passValue) return AssessmentDisplaySets.db.profile.displaySets[passValue[1]].onClickType ~= "PUSH_DISPLAY_SET" end,
					get = displaySetGet,
					set = displaySetSet,
					passValue = "onClickDisplaySet",
					usage = L["Any Tag String"],
				},
				data =
				{
					type = "text",
					name = L["Data"],
					desc = L["On click data."],
					disabled = function(passValue) return AssessmentDisplaySets.db.profile.displaySets[passValue[1]].onClickType == "NONE" end,
					get = displaySetGet,
					set = displaySetSet,
					passValue = "onClickData",
					usage = L["Any Tag String"],
				},
			},
		},
		defaultColor =
		{
			type = "color",
			name = L["Default Color"],
			desc = L["Change the default bar color."],
			get = function(passValue) return unpack(displaySetGet(passValue)) end,
			set = function(passValue, r, g, b, a) displaySetSet(passValue, { r, g, b, a }) end,
			passValue = "defaultColor",
			hasAlpha = true,
		},
		text =
		{
			type = "group",
			name = L["Text"],
			desc = L["Text options."],
			args =
			{
				add =
				{
					type = "execute",
					name = L["Add New Text"],
					desc = L["Adds a new text to the display set."],
					func = function(passValue) AssessmentDisplaySets:AddText(passValue[1]) end,
					passValue = true,
					order = -1
				},
			},
		},
		favorite =
		{
			type = "toggle",
			name = L["Favorite"],
			desc = L["Show this display set as an option when clicking on a window."],
			get = displaySetGet,
			set = function(passValue, v) displaySetSet(passValue, v) AssessmentDisplaySets:UpdateDisplaySetLists() end,
			passValue = "favorite",
		},
		selectable =
		{
			type = "toggle",
			name = L["Selectable"],
			desc = L["Make this display set selectable from window menus."],
			get = displaySetGet,
			set = function(passValue, v) displaySetSet(passValue, v) AssessmentDisplaySets:UpdateDisplaySetLists() end,
			passValue = "selectable",
		},
		copy =
		{
			type = "execute",
			name = L["Copy"],
			desc = L["Copy this display set."],
			func = displaySetCopy,
			passValue = false,
			order = -2,
		},
		remove =
		{
			type = "execute",
			name = L["Remove"],
			desc = L["Remove this display set."],
			func = displaySetRemove,
			passValue = false,
			order = -1,
		},
	}
}

local textOptionsTemplate =
{
	type = "group",
--	name = ["Text %d"] = true,
--	desc = ["Options for text %d"] = true,
	args =
	{
		tag =
		{
			type = "text",
			name = L["Text Tag"],
			desc = L["Select how to display text."],
			get = textGet,
			set = textSet,
			passValue = "tag",
			usage = L["Any Tag String"],
		},
		alignment =
		{
			type = "text",
			name = L["Alignment"],
			desc = L["Select how to align the text."],
			get = textGet,
			set = textSet,
			passValue = "alignment",
			validate = { LEFT = L["Left"], CENTER = L["Center"], RIGHT = L["Right"] },
		},
		alpha =
		{
			type = "range",
			name = L["Alpha"],
			desc = L["Set the text alpha."],
			min = 0,
			max = 1,
			step = 0.01,
			get = textGet,
			set = textSet,
			passValue = "alpha",
		},
		font =
		{
			type = "text",
			name = L["Font"],
			desc = L["Font for text."],
			get = textGet,
			set = textSet,
			passValue = "font",
			validate = SML_font,
		},
		fontSize =
		{
			type = "range",
			name = L["Font Size"],
			desc = L["Font size for text."],
			min = 4,
			max = 32,
			step = 1,
			get = textGet,
			set = textSet,
			passValue = "fontSize",
		},
		shadow =
		{
			type = "toggle",
			name = L["Font Shadow"],
			desc = L["Show font shadow on text"],
			get = textGet,
			set = textSet,
			passValue = "shadow",
		},
		outline =
		{
			type = "text",
			name = L["Outline"],
			desc = L["Sets the outline for text"],
			get = textGet,
			set = textSet,
			passValue = "outline",
			validate = { [""] = L["None"], OUTLINE = L["Normal"], THICKOUTLINE = L["Thick"] },
		},
		offsets =
		{
			type = "group",
			name = L["Offsets"],
			desc = L["Set text offsets."],
			args =
			{
				left =
				{
					type = "range",
					name = L["Left"],
					desc = L["Set the left offset for the text."],
					min = -100,
					max = 100,
					step = 1,
					get = textGet,
					set = textSet,
					passValue = "offsets.left",
				},
				right =
				{
					type = "range",
					name = L["Right"],
					desc = L["Set the right offset for the text."],
					min = -100,
					max = 100,
					step = 1,
					get = textGet,
					set = textSet,
					passValue = "offsets.right",
				},
				vertical =
				{
					type = "range",
					name = L["Vertical"],
					desc = L["Set the vertical offset for the text."],
					min = -32,
					max = 32,
					step = 1,
					get = textGet,
					set = textSet,
					passValue = "offsets.vertical",
				},
			}
		},
		remove =
		{
			type = "execute",
			name = L["Remove"],
			desc = L["Remove this text."],
			func = function(passValue) AssessmentDisplaySets:RemoveText(unpack(passValue[1])) end,
			passValue = false,
			order = -1,
		},
	}
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.Assessment =
{
	titleTag			= "Assessment",
	listType			= "DisplaySetList",
	barValueTag			= "0", 
	filterTag			= "key ~= \""..L["Assessment"].."\"",
	fightIndex			= 0,
	text				= { count = 1, { alignment = "CENTER", tag = "[key:cyan]", offsets = { right = 0 } } },
	description			= L["Assessment"],
	onClickType			= "PUSH_DISPLAY_SET",
	onClickDisplaySet	= "[key]",
	onClickData			= "",
}

local displaySetFavorites = { }
Assessment.ListType.DisplaySetList =
{
	method = function() 
				for k in pairs(displaySetFavorites) do
					displaySetFavorites[k] = nil
				end

				for k, v in pairs(AssessmentDisplaySets.db.profile.displaySets) do
					if v.selectable and v.favorite then
						displaySetFavorites[k] = k
					end
				end
				return pairs(displaySetFavorites) 
			 end,
	description = L["Display Set List"]
}

-----------------------------------------------------------------------------

function AssessmentDisplaySets:CreateValidateLists()
    for k,v in pairs(Assessment.ListType) do
        listTypeValidate[k] = v.description
    end 
            
    for k,v in pairs(Assessment.FilterType) do
        filterTypeValidate[v.tag] = v.description
    end     
end

-----------------------------------------------------------------------------

local function MergeTable(destination, source)
	for k,v in pairs(source) do
		if type(v) == "table" then
			if not destination[k] then
				destination[k] = Assessment:NewTable()
			end
			MergeTable(destination[k], v)
		else
			destination[k] = v
		end
	end
end

local function MergeTableIfEmpty(destination, source)
	for k,v in pairs(source) do
		if type(v) == "table" then
			if not destination[k] then
				destination[k] = Assessment:NewTable()
			end
			MergeTableIfEmpty(destination[k], v)
		elseif not destination[k] then
			destination[k] = v
		end
	end
end

function AssessmentDisplaySets:OnInitialize()
	for k,v in pairs(Assessment.DefaultDisplaySets) do
		local displaySet = self:CopyTable(defaultDisplaySetProfile)
		MergeTable(displaySet, v)
		for i=3,displaySet.text.count do
			MergeTableIfEmpty(displaySet.text[i], defaultTextProfile)
		end
		
		-- Cleanup fields
		displaySet.description = nil
		
		profileDefaults.displaySets[v.description] = displaySet
	end
	
	Assessment:RegisterDefaults("DisplaySets", "profile", profileDefaults)
	
	self:CreateValidateLists()
end

function AssessmentDisplaySets:OnEnable()
	self:CreateOptions()
	self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetListChanged")
end

function AssessmentDisplaySets:CreateTextOptions(passValueData)
	local options = self:CopyOptionsTemplate(textOptionsTemplate, passValueData)
	options.name = L["Text %d"]:format(passValueData[2])
	options.desc = L["Options for text %d"]:format(passValueData[2])
	return options
end

function AssessmentDisplaySets:UpdateDisplaySetValidates()
	for k in pairs(fullDisplaySetValidate) do
		fullDisplaySetValidate[k] = nil
	end

	for k, v in pairs(self.db.profile.displaySets) do
		fullDisplaySetValidate[k] = k
	end
end

function AssessmentDisplaySets:CreateOptions()
	self:UpdateDisplaySetValidates()

	self:ReleaseTable(Assessment.Options.args.displaySets.args, true)
	Assessment.Options.args.displaySets.args = self:CopyTable(displaySetsArgs)
	
	for k in pairs(self.db.profile.displaySets) do
		if Assessment.Options.args.displaySets.args[k] then
			self:ReleaseTable(Assessment.Options.args.displaySets.args[k], true)
		end
		Assessment.Options.args.displaySets.args[k] = self:CreateDisplaySetOptions(k)
	end
end

function AssessmentDisplaySets:CreateDisplaySetOptions(displaySetName)
	local options = self:CopyOptionsTemplate(displaySetOptionsTemplate, displaySetName)
	options.name = displaySetName
	options.desc = L["Options for display set %s"]:format(displaySetName)

	for i = 1, self.db.profile.displaySets[displaySetName].text.count do
		-- We make it a string, otherwise AceConsole can't use it.....
		if options.args.text.args[tostring(i)] then
			self:ReleaseTable(options.args.text.args[tostring(i)], true)
		end
		options.args.text.args[tostring(i)] = self:CreateTextOptions( { displaySetName, i } )
	end
	
	return options
end

-----------------------------------------------------------------------------

StaticPopupDialogs["ASSESSMENT_ADD_DISPLAY_SET"] = 
{
	text = L["Enter a new display set name"],
	button1 = ACCEPT,
	button2 = CANCEL,
	hasEditBox = 1,
	OnAccept = function()
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText()
		AssessmentDisplaySets:AddDisplaySet(text)
	end,
	OnShow = function()
		getglobal(this:GetName().."EditBox"):SetFocus()
	end,
	EditBoxOnEnterPressed = function()
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText()
		AssessmentDisplaySets:AddDisplaySet(text)
		this:GetParent():Hide()
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1,
}

StaticPopupDialogs["ASSESSMENT_COPY_DISPLAY_SET"] = 
{
	text = L["Enter the name to copy this display set to"],
	button1 = ACCEPT,
	button2 = CANCEL,
	hasEditBox = 1,
	OnAccept = function()
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText()
		AssessmentDisplaySets:CopyDisplaySet(StaticPopupDialogs["ASSESSMENT_COPY_DISPLAY_SET"].data, text)
	end,
	OnShow = function()
		getglobal(this:GetName().."EditBox"):SetFocus()
	end,
	EditBoxOnEnterPressed = function()
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText()
		AssessmentDisplaySets:CopyDisplaySet(StaticPopupDialogs["ASSESSMENT_COPY_DISPLAY_SET"].data, text)
		this:GetParent():Hide()
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1,
}

-----------------------------------------------------------------------------

function AssessmentDisplaySets:UpdateDisplaySetLists()
	-- Called when favorite or selectable are changed
	self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetListChanged", displaySetName)
end

function AssessmentDisplaySets:UpdateDisplaySets(displaySetName)
	self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetUpdated", displaySetName)
end

function AssessmentDisplaySets:AddDisplaySet(displaySetName)
	if displaySetName then
		if not self.db.profile.displaySets[displaySetName] then
			self.db.profile.displaySets[displaySetName] = self:CopyTable(defaultDisplaySetProfile)
			self:CreateOptions()
			self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetAdded", displaySetName)
		end
	else
		if Dewdrop then Dewdrop:Close() end
		StaticPopup_Show("ASSESSMENT_ADD_DISPLAY_SET")
	end
end

function AssessmentDisplaySets:CopyDisplaySet(sourceSetName, newDisplaySetName)
	if newDisplaySetName then
		if not self.db.profile.displaySets[newDisplaySetName] then
			self.db.profile.displaySets[newDisplaySetName] = self:CopyTable(self.db.profile.displaySets[sourceSetName])
			self:CreateOptions()
			self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetAdded", newDisplaySetName)
		end
	else
		if Dewdrop then Dewdrop:Close() end
		StaticPopupDialogs["ASSESSMENT_COPY_DISPLAY_SET"].data = sourceSetName
		StaticPopup_Show("ASSESSMENT_COPY_DISPLAY_SET")
	end
end

local removeSetFailReason
function AssessmentDisplaySets:RemoveDisplaySet(displaySetName)
	for k,v in pairs(self.db.profile.displaySets) do

		if v.onClickType == "PUSH_DISPLAY_SET" and v.onClickDisplaySet == displaySetName then
			Assessment:Print(string.format(L["This display set cannot be removed currently because the display set %s references it for on-click."], k))
			return
		end
	end

	removeSetFailReason = nil
	self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetRequestRemove", displaySetName)

	if removeSetFailReason then
		Assessment:Print(removeSetFailReason)
	else
		self.db.profile.displaySets[displaySetName] = nil
		self:CreateOptions()

		self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetRemoved", displaySetName)
	end
end

function AssessmentDisplaySets:PreventDisplaySetRemoval(reason)
	removeSetFailReason = reason
end

function AssessmentDisplaySets:GetDisplaySet(displaySetName)
	return self.db.profile.displaySets[displaySetName]
end

function AssessmentDisplaySets:IterateDisplaySets()
	return pairs(self.db.profile.displaySets)
end

-----------------------------------------------------------------------------

function AssessmentDisplaySets:RemoveText(displaySetName, removeIndex)
	local text = self.db.profile.displaySets[displaySetName].text
	
	for i = removeIndex, text.count do
		text[i] = text[i+1]
	end
	
	text.count = text.count-1

	self:UpdateDisplaySets(displaySetName)

	self:CreateOptions()
end

-----------------------------------------------------------------------------
